Attribute VB_Name = "modImageKit"
'**************************************************
'* Module to encapsulate the Image Kit 6 toolkit: *
'**************************************************

Option Explicit

Public Sub LoadFile(ByVal sFileName As String, ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkFile As IkFile)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If sFileName = vbNullString Then Exit Sub
  
  'Free the previous image data from the memory if there is one:
  If oIkDisp.ImgHandle <> 0 Then
  
    oIkCommon.ImgHandle = oIkDisp.ImgHandle
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    
  End If
  
  'Load the image file:
  oIkFile.FileName = sFileName
  On Error GoTo Error_Load_File
  oIkFile.LoadFile (ikLoad)
  On Error GoTo Error_Handler
  
  oIkCommon.ImgHandle = oIkFile.ImgHandle
  ret = oIkCommon.GetImageType()
  
  If ret = False Then Exit Sub
  
  If oIkCommon.ImageKind <> ikRasterImage Then
  
    'Vector Image:
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    Exit Sub
    
  End If
  
  'Raster Image:
  oIkDisp.ImgHandle = oIkFile.ImgHandle 'Pass the image handle to the display control
  oIkDisp.Display (ikScale)             'Display the image
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:LoadFile", Erl
  Resume Next
  
Error_Load_File:
  MsgBox "Error loading image file '" & oIkFile.FileName & "'.", vbCritical, sAppName
  
End Sub

Public Sub LoadImage(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkFile As IkFile)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkFile.OpenFileDlg = False Then Exit Sub 'Display the Open File dialog to select an image
  
  'Free the previous image data from the memory if there is one:
  If oIkDisp.ImgHandle <> 0 Then
  
    oIkCommon.ImgHandle = oIkDisp.ImgHandle
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    
  End If
  
  'Load the image file:
  oIkFile.LoadFile (ikLoad)
  
  oIkCommon.ImgHandle = oIkFile.ImgHandle
  ret = oIkCommon.GetImageType()
  
  If ret = False Then Exit Sub
  
  If oIkCommon.ImageKind <> ikRasterImage Then
  
    'Vector Image:
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    Exit Sub
    
  End If
  
  'Raster Image:
  oIkDisp.ImgHandle = oIkFile.ImgHandle 'Pass the image handle to the display control
  oIkDisp.Display (ikScale)             'Display the image
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:LoadImage", Erl
  Resume Next
  
End Sub

Public Sub ReloadImage(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkFile As IkFile)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  'Free the previous image data from the memory if there is one:
  If oIkDisp.ImgHandle <> 0 Then
  
    oIkCommon.ImgHandle = oIkDisp.ImgHandle
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    
  Else
  
    Exit Sub
    
  End If
  
  'Reload the image file:
  oIkFile.LoadFile (ikLoad)
  
  oIkCommon.ImgHandle = oIkFile.ImgHandle
  ret = oIkCommon.GetImageType()
  
  If ret = False Then Exit Sub
  
  If oIkCommon.ImageKind <> ikRasterImage Then
  
    'Vector Image:
    oIkCommon.FreeMemory
    oIkDisp.ImgHandle = 0
    Exit Sub
    
  End If
  
  'Raster Image:
  oIkDisp.ImgHandle = oIkFile.ImgHandle 'Pass the image handle to the display control
  oIkDisp.Display (ikScale)             'Display the image
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:ReloadImage", Erl
  Resume Next
  
End Sub

Public Sub SetBrightness(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect, ByVal iBrightness As Integer)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  'Process the total image:
  oIkEffect.SelectMode = 1
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  
  On Error Resume Next
  
  'Execute the RGBLevel method on each color plane that is enabled:
  ret = oIkEffect.RGBLevel(0, 0, 0, iBrightness, iBrightness, iBrightness)
  
  On Error GoTo Error_Handler
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Display the output:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:SetBrightness", Erl
  Resume Next
  
End Sub

Public Sub SetContrast(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect, ByVal iContrast As Integer)

  On Error GoTo Error_Handler
  
  Dim i           As Long
  Dim spx(0 To 4) As Long
  Dim spy(0 To 4) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Set the spline values:
  spx(0) = 0: spy(0) = 0
  spx(1) = 64: spy(1) = spx(1) - iContrast
  spx(2) = 128: spy(2) = 128
  spx(3) = 192: spy(3) = spx(3) + iContrast
  spx(4) = 255: spy(4) = 255
  
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.SelectMode = ikEffectAll
  
  On Error Resume Next
  
  i = oIkEffect.RGBSpline(0, 0, 0, spx(0), spy(0), 5, 1, 1, 1)
  
  On Error GoTo Error_Handler
  
  If i = False Then Exit Sub
  
  'Free the effect control's input image handle:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Display the output:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:SetContrast", Erl
  Resume Next
  
End Sub

Public Sub Mirror1(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  'Make sure there is an image in the Display Control:
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  
  'Set the effect control's input mask handle to 0:
  oIkEffect.InMskHandle = 0
  
  'Execute the reverse the X axis:
  ret = oIkEffect.Rotation(0, True, False, False, 255, 255, 255, False)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Mirror1", Erl
  Resume Next
  
End Sub
    
Public Sub Mirror2(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  'Make sure there is an image in the Display Control:
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  
  'Set the effect control's input mask handle to 0:
  oIkEffect.InMskHandle = 0
  
  'Execute the reverse the Y axis:
  ret = oIkEffect.Rotation(0, False, True, False, 255, 255, 255, False)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Mirror2", Erl
  Resume Next
  
End Sub
    
Public Sub Inverse(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim i           As Long
  Dim spx(0 To 4) As Long
  Dim spy(0 To 4) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Set the spline values:
  spx(0) = 0: spy(0) = 255
  spx(1) = 64: spy(1) = 192
  spx(2) = 128: spy(2) = 128
  spx(3) = 192: spy(3) = 64
  spx(4) = 255: spy(4) = 0
  
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.SelectMode = ikEffectAll
  i = oIkEffect.RGBSpline(0, 0, 0, spx(0), spy(0), 5, 1, 1, 1)
  
  If i = False Then Exit Sub
  
  'Free the effect control's input image handle:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Display the output:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Inverse", Erl
  Resume Next
  
End Sub

Public Sub RedEye(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  oIkEffect.RectTop = oIkDisp.RectTop
  oIkEffect.RectLeft = oIkDisp.RectLeft
  oIkEffect.RectBottom = oIkDisp.RectBottom
  oIkEffect.RectRight = oIkDisp.RectRight
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectEllipse
  If oIkEffect.RectRight And oIkEffect.RectBottom = 0 Then Exit Sub
  ret = oIkEffect.RedEyeRemoval(X(0), Y(0), 4, 95, 90, 120, 50)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Display the output:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:RedEye", Erl
  Resume Next
  
End Sub

Public Sub Sepia(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle to the effect control and convert to grayscale:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  ret = oIkEffect.ConvertColor(80, False, False, 128)
  
  'If successful, clear the effect control's input image handle:
  If ret <> 0 Then
  
    oIkCommon.ImgHandle = oIkEffect.InImgHandle
    oIkCommon.FreeMemory
    oIkEffect.InImgHandle = oIkEffect.OutImgHandle
    
  Else 'Grayscale, scanned image
  
    oIkEffect.InImgHandle = oIkDisp.ImgHandle
    
  End If
  
  ret = oIkEffect.ConvertColor(24, False, False, 128)
  
  'If successful, clear the effect control's input image handle:
  If ret <> 0 Then
  
    oIkCommon.ImgHandle = oIkEffect.InImgHandle
    oIkCommon.FreeMemory
    
  End If
  
  'Process the original (gray) image:
  oIkCommon.ImgHandle = oIkEffect.OutImgHandle
  oIkEffect.InImgHandle = oIkCommon.CopyImage
  
  'Process the total image:
  oIkEffect.SelectMode = 1
  
  'Execute the RGBLevel method on each color plane to produce sepia:
  ret = oIkEffect.RGBLevel(0, 0, 0, 40, -20, -80)
  
  'Free the effect control's input image handle:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Display (ikScale)
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Sepia", Erl
  Resume Next
  
End Sub

Public Sub Gray(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Adjust the image chroma:
  oIkEffect.InOut = ikInside
'  oIkEffect.SelectMode = ikEffectPolygon
  
'  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then
  
    oIkEffect.SelectMode = ikEffectAll
    
'  End If
  
  ret = oIkEffect.Chroma(X(0), Y(0), 4, -100)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Gray", Erl
  Resume Next
  
End Sub

Public Sub Blur(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Blur the image:
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then oIkEffect.SelectMode = ikEffectAll
  ret = oIkEffect.Blur(X(0), Y(0), 4, 5)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Blur", Erl
  Resume Next
  
End Sub

Public Sub MotionBlur(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Perform a motion blur effect:
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then oIkEffect.SelectMode = ikEffectAll
  ret = oIkEffect.MotionBlur(X(0), Y(0), 4, 2, 10, 20)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:MotionBlur", Erl
  Resume Next
  
End Sub

Public Sub Waves(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Perform a wave-like effect:
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then oIkEffect.SelectMode = ikEffectAll
  ret = oIkEffect.Waves(X(0), Y(0), 4, 0, 0, 100, 10, 0.5, 1, 0, 0, 0)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Waves", Erl
  Resume Next
  
End Sub

Public Sub Ripple(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Perform a ripple-like effect:
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then oIkEffect.SelectMode = ikEffectAll
  ret = oIkEffect.Ripple(X(0), Y(0), 4, 20, 10, 0.5, False, False, 0, 0, 0)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Ripple", Erl
  Resume Next
  
End Sub

Public Sub Canvas(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret       As Boolean
  Dim X(0 To 3) As Long
  Dim Y(0 To 3) As Long
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  X(0) = oIkDisp.RectLeft: Y(0) = oIkDisp.RectTop
  X(1) = oIkDisp.RectRight: Y(1) = oIkDisp.RectTop
  X(2) = oIkDisp.RectRight: Y(2) = oIkDisp.RectBottom
  X(3) = oIkDisp.RectLeft: Y(3) = oIkDisp.RectBottom
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Make the image look like a canvas:
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  If oIkDisp.RectLeft = 0 And oIkDisp.RectTop = 0 And oIkDisp.RectRight = 0 And oIkDisp.RectBottom = 0 Then oIkEffect.SelectMode = ikEffectAll
  ret = oIkEffect.Canvas(X(0), Y(0), 4, 1, 10)
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Canvas", Erl
  Resume Next
  
End Sub

Public Sub Rotate1(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Rotate the image:
  ret = oIkEffect.Rotation(-90 * 100, False, False, False, 255, 255, 255, False)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Rotate1", Erl
  Resume Next
  
End Sub

Public Sub Rotate2(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)
    
  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Rotate the image:
  ret = oIkEffect.Rotation(90 * 100, False, False, False, 255, 255, 255, False)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Rotate2", Erl
  Resume Next
  
End Sub

Public Sub Rotate(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect, ByVal fDegrees As Double, ByVal iR As Integer, ByVal iG As Integer, ByVal iB As Integer)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Rotate the image:
  ret = oIkEffect.Rotation(fDegrees * 100, False, False, True, iR, iG, iB, False)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Rotate", Erl
  Resume Next
  
End Sub

Public Sub Resize(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect, ByVal lWidthPixels As Long, ByVal lHeightPixels As Long)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Resize the image:
  
  ret = oIkEffect.Resize(lWidthPixels, lHeightPixels, True)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Resize", Erl
  Resume Next
  
End Sub

Public Sub Crop(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  'Specify the area to be cropped:
  oIkEffect.RectLeft = oIkDisp.RectLeft
  oIkEffect.RectTop = oIkDisp.RectTop
  oIkEffect.RectRight = oIkDisp.RectRight
  oIkEffect.RectBottom = oIkDisp.RectBottom
  
  On Error Resume Next
  
  'Crop the selection:
  ret = oIkEffect.CutRectImage
  
  On Error GoTo Error_Handler
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Crop", Erl
  Resume Next
  
End Sub

Public Sub Ellipse(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectEllipse
  
  'Specify the area to be cropped:
  oIkEffect.RectBottom = oIkDisp.RectBottom
  oIkEffect.RectLeft = oIkDisp.RectLeft
  oIkEffect.RectRight = oIkDisp.RectRight
  oIkEffect.RectTop = oIkDisp.RectTop
  
  'Select an ellipse:
  ret = oIkEffect.SelectImage(0, 0, 0, 255, 255, 255)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Ellipse", Erl
  Resume Next
  
End Sub

Public Sub Polygon(ByRef oIkCommon As IkCommon, ByRef oIkDisp As IkDisp, ByRef oIkEffect As IkEffect, ByRef Ix() As Long, ByRef Iy() As Long, ByVal Points As Long)

  On Error GoTo Error_Handler
  
  Dim ret As Boolean
  
  If oIkDisp.ImgHandle = 0 Then Exit Sub
  
  'Pass the image handle from the display control to the effect control's input image handle:
  oIkEffect.InImgHandle = oIkDisp.ImgHandle
  oIkEffect.InMskHandle = 0
  
  oIkEffect.InOut = ikInside
  oIkEffect.SelectMode = ikEffectPolygon
  
  'Specify the area to be cropped:
  oIkEffect.RectBottom = oIkDisp.RectBottom
  oIkEffect.RectLeft = oIkDisp.RectLeft
  oIkEffect.RectRight = oIkDisp.RectRight
  oIkEffect.RectTop = oIkDisp.RectTop
  
  'Select a polygon:
  ret = oIkEffect.SelectImage(Ix(0), Iy(0), Points, 255, 255, 255)
  
  If ret = False Then Exit Sub
  
  'Free the effect control's input image handle from the memory:
  oIkCommon.ImgHandle = oIkEffect.InImgHandle
  oIkCommon.FreeMemory
  
  'Pass the effect control's output image handle to the display control and display the image:
  oIkDisp.ImgHandle = oIkEffect.OutImgHandle
  oIkDisp.Refresh
  
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:Polygon", Erl
  Resume Next
  
End Sub

Public Sub SaveAsJPEG(ByVal sFileName As String, ByVal iQuality As Integer, ByRef oIkCommon As IkCommon, ByRef oIkFile As IkFile, ByRef oIkDisp As IkDisp)

  On Error GoTo Error_Handler
  oIkCommon.ImgHandle = oIkDisp.ImgHandle
  oIkCommon.GetImageType
  oIkFile.ImgHandle = oIkCommon.ImgHandle
  oIkFile.FileName = sFileName
  oIkFile.Comment = vbNullString
  oIkFile.JpegQuality = iQuality
  oIkFile.JpegSubsamp = 0 '0 => 4:1:1 / 1 => 4:2:2 / 2 => 4:4:4
  On Error GoTo Error_Save_JPEG
  oIkFile.SaveFile (ikSaveJPEG)
  On Error GoTo Error_Handler
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:SaveAsJPEG", Erl
  Resume Next
  
Error_Save_JPEG:
  MsgBox "Error saving JPEG file '" & oIkFile.FileName & "'.", vbCritical, sAppName
  
End Sub

Public Sub SaveAsGIF(ByVal sFileName As String, ByRef oIkCommon As IkCommon, ByRef oIkFile As IkFile, ByRef oIkDisp As IkDisp)

  On Error GoTo Error_Handler
  oIkCommon.ImgHandle = oIkDisp.ImgHandle
  oIkCommon.GetImageType
  oIkFile.ImgHandle = oIkCommon.ImgHandle
  oIkFile.FileName = sFileName
  oIkFile.Comment = vbNullString
  On Error GoTo Error_Save_GIF
  oIkFile.SaveFile (ikSaveGIF)
  On Error GoTo Error_Handler
  Exit Sub
  
Error_Handler:
  HandleError Err, "modImageKit:SaveAsGIF", Erl
  Resume Next
  
Error_Save_GIF:
  MsgBox "Error saving GIF file '" & oIkFile.FileName & "'.", vbCritical, sAppName
  
End Sub
